@@ -10,11 +10,11 @@ FSM::Basic -  Finite state machine using HASH as state definitions
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -128,8 +128,8 @@ sub new
 {
     my ( $class, $l, $s ) = @_;
     my $self;
-    $self->{states_list} = $l;
-    $self->{state}       = $s;
+    $self->{ states_list } = $l;
+    $self->{ state }       = $s;
     bless( $self, $class );
     return $self;
 }
@@ -147,107 +147,111 @@ sub run
 {
     my ( $self, $in ) = @_;
 
-    no autovivification;
     my $output = '';
-    if ( exists $self->{states_list}{$self->{state}}{repeat} && $self->{states_list}{$self->{state}}{repeat} <= 0 )
+    if ( exists $self->{ states_list } )
     {
-        $self->{previous_state} = $self->{state};
-        $self->{state} = $self->{states_list}{$self->{state}}{expect}{not_matching0} // $self->{states_list}{$self->{state}}{not_matching0};
-        if ( exists $self->{states_list}{$self->{previous_state}}{not_matching_info_last} )
+        if (   exists $self->{ states_list }{ $self->{ state } }
+            && exists $self->{ states_list }{ $self->{ state } }{ repeat }
+            && $self->{ states_list }{ $self->{ state } }{ repeat } <= 0 )
         {
-            $output = $self->{states_list}{$self->{previous_state}}{not_matching_info_last};
-        }
-        $output .= $self->{states_list}{$self->{state}}{output} // '';
-        return ( $self->{states_list}{$self->{state}}{final} // 0, $output );
-    }
-    if ( exists $self->{states_list}{$self->{state}}{expect} )
-    {
-        if ( exists $self->{states_list}{$self->{state}}{info} )
-        {
-            $output = $self->{states_list}{$self->{state}}{info} . $output;
-        }
-        if ( exists $self->{states_list}{$self->{state}}{info_once} )
-        {
-            $output = delete( $self->{states_list}{$self->{state}}{info_once} ) . $output;
-        }
-        my $state;
-        if (exists $self->{previous_output} && $in eq '' && $self->{previous_output} =~ /\[(.+)\]/ )
-        {
-            $in = $1;
-        }
-        if ( exists $self->{states_list}{$self->{state}}{expect}{$in} )
-        {
-            $state = $self->{states_list}{$self->{state}}{expect}{$in};
-        }
-        else
-        {
-            foreach my $key ( keys %{$self->{states_list}{$self->{state}}{expect}} )
+            $self->{ previous_state } = $self->{ state };
+            $self->{ state } = $self->{ states_list }{ $self->{ state } }{ expect }{ not_matching0 } // $self->{ states_list }{ $self->{ state } }{ not_matching0 };
+            if ( exists $self->{ states_list }{ $self->{ previous_state } }{ not_matching_info_last } )
             {
-                if ( $in =~ /$key/ )
+                $output = $self->{ states_list }{ $self->{ previous_state } }{ not_matching_info_last };
+            }
+            $output .= $self->{ states_list }{ $self->{ state } }{ output } // '';
+            return ( $self->{ states_list }{ $self->{ state } }{ final } // 0, $output );
+        }
+        if ( exists $self->{ states_list }{ $self->{ state } }{ expect } )
+        {
+            if ( exists $self->{ states_list }{ $self->{ state } }{ info } )
+            {
+                $output = $self->{ states_list }{ $self->{ state } }{ info } . $output;
+            }
+            if ( exists $self->{ states_list }{ $self->{ state } }{ info_once } )
+            {
+                $output = delete( $self->{ states_list }{ $self->{ state } }{ info_once } ) . $output;
+            }
+            my $state;
+            if (   exists $self->{ previous_output }
+                && $in eq ''
+                && $self->{ previous_output } =~ /\[(.+)\]/ )
+            {
+                $in = $1;
+            }
+            if ( exists $self->{ states_list }{ $self->{ state } }{ expect }{ $in } )
+            {
+                $state = $self->{ states_list }{ $self->{ state } }{ expect }{ $in };
+            }
+            else
+            {
+                foreach my $key ( keys %{ $self->{ states_list }{ $self->{ state } }{ expect } } )
                 {
-                    $state = $self->{states_list}{$self->{state}}{expect}{$key};
+                    if ( $in =~ /$key/ )
+                    {
+                        $state = $self->{ states_list }{ $self->{ state } }{ expect }{ $key };
+                    }
                 }
             }
+            if ( ref $state eq 'HASH' )
+            {
+                $self->{ previous_state }  = $self->{ state };
+                $self->{ previous_output } = $state->{ output } // $self->{ states_list }{ $self->{ state } }{ output } // '';
+                $self->{ state }           = $state->{ matching } // $self->{ state };
+                $output .= $state->{ output } // $self->{ states_list }{ $self->{ state } }{ output } // '';
+                if ( exists $state->{ cmd } )
+                {
+                    my $cmd_state = delete $state->{ cmd };
+                    $cmd_state =~ s/\$in/$in/g;
+                    push( @{ $self->{ cmd_stack } }, $cmd_state );
+                }
+                if ( exists $state->{ cmd_exec } )
+                {
+                    my $cmd_exec = join ' ', @{ $self->{ cmd_stack } };
+                    $output = `$cmd_exec` . $output;
+                    $self->{ cmd_exec } = [];
+                }
+                if ( exists $state->{ exec } )
+                {
+                    my $old_exec = $state->{ exec };
+                    $state->{ exec } =~ s/__IN__/$in/g;
+                    $output = `$state->{exec}` . $output;
+                    $state->{ exec } = $old_exec;
+                }
+                if ( exists $state->{ do } )
+                {
+                    my $old_do = $state->{ do };
+                    $state->{ do } =~ s/__IN__/$in/g;
+                    $output = ( eval $state->{ do } ) . $output;
+                    $state->{ do } = $old_do;
+                }
+            }
+            else
+            {
+                $self->{ previous_state } = $self->{ state };
+                $self->{ state } = $self->{ states_list }{ $self->{ state } }{ not_matching } // $self->{ state };
+                $self->{ states_list }{ $self->{ state } }{ repeat }--
+                  if exists $self->{ states_list }{ $self->{ state } }{ repeat };
+                $output .= $self->{ states_list }{ $self->{ state } }{ output } // '';
+                if ( exists $self->{ states_list }{ $self->{ state } }{ not_matching_info } )
+                {
+                    $output = $self->{ states_list }{ $self->{ state } }{ not_matching_info } . "\n" . $output;
+                }
+                return ( $self->{ states_list }{ $self->{ state } }{ $in }{ final } // $self->{ states_list }{ $self->{ state } }{ final } // 0, $output );
+            }
+            return ( $self->{ states_list }{ $self->{ state } }{ $in }{ final } // $self->{ states_list }{ $self->{ state } }{ final } // 0, $output );
         }
-        if ( ref $state eq 'HASH' )
-        {
-            $self->{previous_state}  = $self->{state};
-            $self->{previous_output} = $state->{output} // $self->{states_list}{$self->{state}}{output} // '';
-            $self->{state}           = $state->{matching} // $self->{state};
-            $output .= $state->{output} // $self->{states_list}{$self->{state}}{output} // '';
-            if ( exists $state->{cmd} )
-            {
-                my $cmd_state = delete $state->{cmd};
-                $cmd_state =~ s/\$in/$in/g;
-                push( @{$self->{cmd_stack}}, $cmd_state );
-            }
-            if ( exists $state->{cmd_exec} )
-            {
-                my $cmd_exec = join ' ', @{$self->{cmd_stack}};
-                $output = `$cmd_exec` . $output;
-                $self->{cmd_exec} = [];
-            }
-            if ( exists $state->{exec} )
-            {
-                my $old_exec = $state->{exec};
-                $state->{exec} =~ s/__IN__/$in/g;
-                $output = `$state->{exec}` . $output;
-                $state->{exec} = $old_exec;
-            }
-            if ( exists $state->{do} )
-            {
-                my $old_do = $state->{do};
-                $state->{do} =~ s/__IN__/$in/g;
-                $output = ( eval $state->{do} ) . $output;
-                $state->{do} = $old_do;
-            }
-        }
-        else
-        {
-            $self->{previous_state} = $self->{state};
-            $self->{state} = $self->{states_list}{$self->{state}}{not_matching} // $self->{state};
-            $self->{states_list}{$self->{state}}{repeat}-- if exists $self->{states_list}{$self->{state}}{repeat};
-            $output .= $self->{states_list}{$self->{state}}{output} // '';
-            if ( exists $self->{states_list}{$self->{state}}{not_matching_info} )
-            {
-                $output = $self->{states_list}{$self->{state}}{not_matching_info} . "\n" . $output;
-            }
-            return ( $self->{states_list}{$self->{state}}{$in}{final} // $self->{states_list}{$self->{state}}{final} // 0, $output );
-        }
-        return ( $self->{states_list}{$self->{state}}{$in}{final} // $self->{states_list}{$self->{state}}{final} // 0, $output );
     }
-    #    else
-    #    {
-    #        return ( 3, $output );
-    #    }
 }
 
 sub set
 {
     my ( $self, $in ) = @_;
-    $self->{previous_state}  = $self->{state};
-    $self->{previous_output} = $self->{states_list}{$self->{state}}{output} // '';
-    $self->{state}           = $in if exists $self->{states_list}{$in};
+    $self->{ previous_state }  = $self->{ state };
+    $self->{ previous_output } = $self->{ states_list }{ $self->{ state } }{ output } // '';
+    $self->{ state }           = $in
+      if exists $self->{ states_list }{ $in };
 }
 
 #sub edit
@@ -277,7 +281,7 @@ sub set
 #                }
 #            }
 #        }
-#        
+#
 #
 #    return $state;
 #}
